package webui

import (
	"net/http"
)

// Sets generally useful HTTP headers, e.g. limiting frame-options.
func setDefaultHeaders(w http.ResponseWriter) {

	w.Header().Set("X-Content-Type-Options", "nosniff")
	w.Header().Set("X-Frame-Options", "SAMEORIGIN")
	w.Header().Set("X-XSS-Protection", "1; mode=block")
	w.Header().Set("Access-Control-Allow-Methods", "GET, POST, OPTIONS")
    w.Header().Set("Access-Control-Allow-Origin", "*")
    w.Header().Set("Access-Control-Expose-Headers", "Content-Type")
	// w.Header().Set("Strict-Transport-Security", "max-age=63072000")

}

// Sets the HTTP headers expected to be returned when serving HTML pages.
func setHeadersForHtml(w http.ResponseWriter) {
    setDefaultHeaders(w)
	w.Header().Set("Content-Security-Policy", "default-src 'self'; font-src 'self'; object-src 'none'; frame-src 'self'; frame-ancestors 'none'; base-uri 'none'; script-src 'self'; style-src 'self' 'unsafe-inline'")
}

// Sets the HTTP headers expected to be returned when serving JSON.
func setHeadersForJson(w http.ResponseWriter) {
    setDefaultHeaders(w)
    w.Header().Set("Content-Type", "application/json")
}

// Sets the HTTP headers expected to be returned when serving CSS files.
func setHeadersForCss(w http.ResponseWriter) {
    setDefaultHeaders(w)
    w.Header().Set("Content-Type", "text/css")
}

// Sets the HTTP headers expected to be returned when serving JS files.
func setHeadersForJs(w http.ResponseWriter) {
    setDefaultHeaders(w)
    w.Header().Set("Content-Type", "text/javascript")
}

// Sets the HTTP headers expected to be returned when serving SVG files.
func setHeadersForSvg(w http.ResponseWriter) {
    setDefaultHeaders(w)
    w.Header().Set("Content-Type", "image/svg+xml")
}
func setHeadersForEventStream(w http.ResponseWriter) {
    setDefaultHeaders(w)
    w.Header().Set("X-Accel-Buffering", "no")
    w.Header().Set("Content-Type", "text/event-stream")
    w.Header().Set("Cache-Control", "no-cache")
    w.Header().Set("Connection", "keep-alive")
}
